#!/usr/local/bin/perl

#- this package implements a recursive directory walk
#	calling callback routines for each directory and
#	each matching file encountered in the walk.

# entry point: walk(path, filter)
# callbacks: foreachDirectory(path), foreachFile(path, filename)

# for compatibility with scripts written for the
# older version of this framework, the entry point
# "procdir(path, filter)" and the callback routine
# "procfile(path, filename)" are still supported
# a callback to procfile() is only attempted if
# the preferred callback routine foreachFile() has
# not been defined.


#- the "foreachXxx()" functions are
#	to be implemented by the scripts
#	using the treewalk skeleton, here
#	are the templates for them:
#
#sub foreachDirectory {	# path
#	local($path) = @_;
#	print " " x $lvl, "def foreachDir: ", $path . "\n" ;
#}
#
#sub foreachFile {	# path, filename
#	local($path,$f) = @_;
#	print " " x $lvl, "def foreachFile: ", $path . "/" . $f . "\n" ;
#}

#- the "procfile()" function is still called
#	for backwards compatibility with 
#	older scripts that implemented 
#	procfile() instead of foreachFile()
#
#sub procfile {
#	local ($path, $f) = @_;
#	print " " x $lvl, "def procfile: ", $path . "\n" ;
#}

#- kept for compatibility w/ older scripts
sub procdir {
	local($path, $filtr) = @_;
	walk($path, $filtr);
}

#- globals
$lvl = 0 ;

#- main entry point
sub walk {
	local($path, $filtr) = @_;
	local($DH, @dlist, @flist, $d, $f);

	#print " " x $lvl, "walk: ", $path . "\n" ;
	$lvl = $lvl + 1 ;
	@flist = ();
	@dlist = ();

	if (opendir(DH, $path)) {
#		print " " x $lvl . "directory: " . $path . "\n";

		if (defined(&foreachDirectory)) {
			do foreachDirectory($path) ;
		}

		@list = readdir(DH);
		closedir(DH);


		# dlist = ( $path );
		# flist = ( $path );

		#print "list: ", @list, "\n";

		foreach $x (@list) {
			next if $x eq "." ;
			next if $x eq ".." ;

			#print "!", $x, "!$filtr!\n";
			$fp = $path . "/" . $x;

			if (-f $fp && $x =~ m/$filtr/i)	{ 
				#print "f=" . $fp . "\n";
				$flist[$#flist+1] = $x; 
			}

			if (-d $fp) {
				#print "d=" . $fp . "\n";
				$dlist[$#dlist+1] = $x ; 
			}
		}


		foreach $f (@flist) {
			if (defined(&foreachFile)) {
				do foreachFile($path, $f);
			} elsif (defined(&procfile)) {
				do procfile($path, $f) ;
			}
		}

		foreach $d (@dlist) {
			#print "d=", $path . "/" . $d . "\n";
			do walk($path . "/" . $d, $filtr) ;
		}
	} else {
		print " " x $lvl, "$path empty\n" ;
	}

	$lvl = $lvl - 1;
}

#$root = ".";
#$root = $ARGV[0] if defined($ARGV[0]);
#walk($root, "*");

1;	#- return value so "require 'treewalk.pl' " works

